@@ -29,17 +29,13 @@ import sys
 
 
 def getSupportedPythonVersions():
-    """ Officially supported Python versions for Nuitka.
-
-    """
+    """Officially supported Python versions for Nuitka."""
 
     return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8")
 
 
 def getPartiallySupportedPythonVersions():
-    """ Partially supported Python versions for Nuitka.
-
-    """
+    """Partially supported Python versions for Nuitka."""
 
     return ()
 
@@ -79,8 +75,8 @@ def isAtLeastSubVersion(version):
 
 
 def getErrorMessageExecWithNestedFunction():
-    """ Error message of the concrete Python in case an exec occurs in a
-        function that takes a closure variable.
+    """Error message of the concrete Python in case an exec occurs in a
+    function that takes a closure variable.
     """
 
     assert python_version < 300
@@ -108,7 +104,10 @@ def getComplexCallSequenceErrorTemplate():
             f(*None)
         except TypeError as e:
             result = (
-                e.args[0].replace("NoneType object", "%s").replace("NoneType", "%s")
+                e.args[0]
+                .replace("NoneType object", "%s")
+                .replace("NoneType", "%s")
+                .replace("None ", "%s ")
             )
             getComplexCallSequenceErrorTemplate.result = result
         else:
@@ -253,3 +252,16 @@ def getPythonABI():
         abiflags = ""
 
     return abiflags
+
+
+def getSystemPrefixPath():
+    """Return real sys.prefix as an absolute path.
+
+    Note: This breaks out of virtualenvs as necessary.
+
+    Returns: path to system prefix
+    """
+
+    sys_prefix = getattr(sys, "real_prefix", getattr(sys, "base_prefix", sys.prefix))
+    sys_prefix = os.path.abspath(sys_prefix)
+    return sys_prefix
