@@ -31,6 +31,12 @@ from __future__ import print_function
 import logging
 import os
 import sys
+from contextlib import contextmanager
+
+from nuitka.utils.ThreadedExecutor import RLock
+
+# Written by Options module.
+is_quiet = False
 
 
 def printIndented(level, *what):
@@ -49,8 +55,9 @@ def printError(message):
     print(message, file=sys.stderr)
 
 
-def flushStdout():
+def flushStandardOutputs():
     sys.stdout.flush()
+    sys.stderr.flush()
 
 
 def getEnableStyleCode(style):
@@ -95,38 +102,58 @@ def getDisableStyleCode():
     return "\033[0m"
 
 
+# Locking seems necessary to avoid colored output split up.
+trace_lock = RLock()
+
+
+@contextmanager
+def withTraceLock():
+    """ Hold a lock, so traces cannot be output at the same time mixing them up. """
+
+    trace_lock.acquire()
+    yield
+    trace_lock.release()
+
+
 def my_print(*args, **kwargs):
-    """ Make sure we flush after every print.
+    """Make sure we flush after every print.
 
     Not even the "-u" option does more than that and this is easy enough.
 
     Use kwarg style=[option] to print in a style listed below
     """
+    with withTraceLock():
+        if "style" in kwargs:
+            style = kwargs["style"]
+            del kwargs["style"]
 
-    if "style" in kwargs:
-        style = kwargs["style"]
-        del kwargs["style"]
+            if "end" in kwargs:
+                end = kwargs["end"]
+                del kwargs["end"]
+            else:
+                end = "\n"
 
-        if style is not None and sys.stdout.isatty():
-            enable_style = getEnableStyleCode(style)
+            if style is not None and sys.stdout.isatty():
+                enable_style = getEnableStyleCode(style)
 
-            if enable_style is None:
-                raise ValueError(
-                    "%r is an invalid value for keyword argument style" % style
-                )
+                if enable_style is None:
+                    raise ValueError(
+                        "%r is an invalid value for keyword argument style" % style
+                    )
 
-            _enableAnsi()
+                _enableAnsi()
 
-            print(enable_style, end="")
+                print(enable_style, end="", **kwargs)
 
-        print(*args, **kwargs)
+            print(*args, end=end, **kwargs)
 
-        if style is not None and sys.stdout.isatty():
-            print(getDisableStyleCode(), end="")
-    else:
-        print(*args, **kwargs)
+            if style is not None and sys.stdout.isatty():
+                print(getDisableStyleCode(), end="", **kwargs)
+        else:
+            print(*args, **kwargs)
 
-    flushStdout()
+        # Flush the output.
+        kwargs.get("file", sys.stdout).flush()
 
 
 # TODO: Stop using logging at all, and only OurLogger.
@@ -138,20 +165,60 @@ class OurLogger(object):
         self.name = name
         self.base_style = base_style
 
+    def my_print(self, message, **kwargs):
+        # For overload, pylint: disable=no-self-use
+        my_print(message, **kwargs)
+
     def warning(self, message, style="red"):
         message = "%s:WARNING: %s" % (self.name, message)
 
         style = style or self.base_style
-        my_print(message, style=style)
+        self.my_print(message, style=style, file=sys.stderr)
 
     def info(self, message, style=None):
-        message = "%s:INFO: %s" % (self.name, message)
+        if not is_quiet:
+            message = "%s:INFO: %s" % (self.name, message)
 
-        style = style or self.base_style
-        my_print(message, style=style)
+            style = style or self.base_style
+            self.my_print(message, style=style)
+
+
+class FileLogger(OurLogger):
+    def __init__(self, name, base_style=None, file_handle=sys.stdout):
+        OurLogger.__init__(self, name=name, base_style=base_style)
+
+        self.file_handle = file_handle
+
+    def my_print(self, message, **kwargs):
+        message = message + "\n"
+
+        self.file_handle.write(message)
+        self.file_handle.flush()
+
+    def setFileHandle(self, file_handle):
+        self.file_handle = file_handle
+
+    def info(self, message, style=None):
+        if not is_quiet or self.file_handle is not sys.stdout:
+            message = "%s:INFO: %s" % (self.name, message)
+
+            style = style or self.base_style
+            self.my_print(message, style=style)
+
+    def debug(self, message, style=None):
+        if self.file_handle is not sys.stdout:
+            message = "%s:DEBUG: %s" % (self.name, message)
+
+            style = style or self.base_style
+            self.my_print(message, style=style)
 
 
 general = OurLogger("Nuitka")
 codegen_missing = OurLogger("Nuitka-codegen-missing")
 plugins_logger = OurLogger("Nuitka-Plugins")
 recursion_logger = OurLogger("Nuitka-Recursion")
+dependencies_logger = OurLogger("Nuitka-Dependencies")
+optimization_logger = FileLogger("Nuitka-Optimization")
+codegen_logger = OurLogger("Nuitka-Codegen")
+inclusion_logger = FileLogger("Nuitka-Inclusion")
+scons_logger = OurLogger("Nuitka-Scons")
