@@ -34,8 +34,6 @@ work on, and let them decide and do the heavy lifting of optimization
 and annotation is happening in the nodes that implement these compute slots.
 """
 
-from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
-
 from .ExpressionBases import (
     ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
@@ -45,14 +43,14 @@ from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 
 
 class StatementAssignmentAttribute(StatementChildrenHavingBase):
-    """ Assignment to an attribute.
+    """Assignment to an attribute.
 
-        Typically from code like: source.attribute_name = expression
+    Typically from code like: source.attribute_name = expression
 
-        Both source and expression may be complex expressions, the source
-        is evaluated first. Assigning to an attribute has its on slot on
-        the source, which gets to decide if it knows it will work or not,
-        and what value it will be.
+    Both source and expression may be complex expressions, the source
+    is evaluated first. Assigning to an attribute has its on slot on
+    the source, which gets to decide if it knows it will work or not,
+    and what value it will be.
     """
 
     kind = "STATEMENT_ASSIGNMENT_ATTRIBUTE"
@@ -97,13 +95,13 @@ class StatementAssignmentAttribute(StatementChildrenHavingBase):
 
 
 class StatementDelAttribute(StatementChildHavingBase):
-    """ Deletion of an attribute.
+    """Deletion of an attribute.
 
-        Typically from code like: del source.attribute_name
+    Typically from code like: del source.attribute_name
 
-        The source may be complex expression. Deleting an attribute has its on
-        slot on the source, which gets to decide if it knows it will work or
-        not, and what value it will be.
+    The source may be complex expression. Deleting an attribute has its on
+    slot on the source, which gets to decide if it knows it will work or
+    not, and what value it will be.
     """
 
     kind = "STATEMENT_DEL_ATTRIBUTE"
@@ -145,9 +143,9 @@ class StatementDelAttribute(StatementChildHavingBase):
 
 
 class ExpressionAttributeLookup(ExpressionChildHavingBase):
-    """ Looking up an attribute of an object.
+    """Looking up an attribute of an object.
 
-        Typically code like: source.attribute_name
+    Typically code like: source.attribute_name
     """
 
     kind = "EXPRESSION_ATTRIBUTE_LOOKUP"
@@ -190,12 +188,12 @@ class ExpressionAttributeLookup(ExpressionChildHavingBase):
 
 
 class ExpressionAttributeLookupSpecial(ExpressionAttributeLookup):
-    """ Special lookup up an attribute of an object.
+    """Special lookup up an attribute of an object.
 
-        Typically from code like this: with source: pass
+    Typically from code like this: with source: pass
 
-        These directly go to slots, and are performed for with statements
-        of Python2.7 or higher.
+    These directly go to slots, and are performed for with statements
+    of Python2.7 or higher.
     """
 
     kind = "EXPRESSION_ATTRIBUTE_LOOKUP_SPECIAL"
@@ -209,11 +207,11 @@ class ExpressionAttributeLookupSpecial(ExpressionAttributeLookup):
 
 
 class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
-    """ Built-in "getattr".
+    """Built-in "getattr".
 
-        Typical code like this: getattr(object_arg, name, default)
+    Typical code like this: getattr(object_arg, name, default)
 
-        The default is optional, but computed before the lookup is done.
+    The default is optional, but computed before the lookup is done.
     """
 
     kind = "EXPRESSION_BUILTIN_GETATTR"
@@ -222,11 +220,10 @@ class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
     getAttribute = ExpressionChildrenHavingBase.childGetter("name")
     getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
-    @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, object_arg, name, default, source_ref):
+    def __init__(self, expression, name, default, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values={"expression": object_arg, "name": name, "default": default},
+            values={"expression": expression, "name": name, "default": default},
             source_ref=source_ref,
         )
 
@@ -272,9 +269,9 @@ attribute '%s' to mere attribute lookup"""
 
 
 class ExpressionBuiltinSetattr(ExpressionChildrenHavingBase):
-    """ Built-in "setattr".
+    """Built-in "setattr".
 
-        Typical code like this: setattr(source, attribute, value)
+    Typical code like this: setattr(source, attribute, value)
     """
 
     kind = "EXPRESSION_BUILTIN_SETATTR"
@@ -283,11 +280,10 @@ class ExpressionBuiltinSetattr(ExpressionChildrenHavingBase):
     getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
     getValue = ExpressionChildrenHavingBase.childGetter("value")
 
-    @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, object_arg, name, value, source_ref):
+    def __init__(self, expression, name, value, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values={"expression": object_arg, "attribute": name, "value": value},
+            values={"expression": expression, "attribute": name, "value": value},
             source_ref=source_ref,
         )
 
@@ -304,11 +300,10 @@ class ExpressionBuiltinHasattr(ExpressionChildrenHavingBase):
     named_children = ("expression", "attribute")
     getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
 
-    @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, object_arg, name, source_ref):
+    def __init__(self, expression, name, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values={"expression": object_arg, "attribute": name},
+            values={"expression": expression, "attribute": name},
             source_ref=source_ref,
         )
 
@@ -359,14 +354,16 @@ class ExpressionAttributeCheck(ExpressionChildHavingBase):
 
     __slots__ = ("attribute_name",)
 
-    @calledWithBuiltinArgumentNamesDecorator
-    def __init__(self, object_arg, attribute_name, source_ref):
+    def __init__(self, expression, attribute_name, source_ref):
         ExpressionChildHavingBase.__init__(
-            self, value=object_arg, source_ref=source_ref
+            self, value=expression, source_ref=source_ref
         )
 
         self.attribute_name = attribute_name
 
+    def getDetails(self):
+        return {"attribute_name": self.attribute_name}
+
     def computeExpression(self, trace_collection):
         # We do at least for compile time constants optimization here, but more
         # could be done, were we to know shapes.
