@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,7 @@
 
 from .ExpressionBases import (
     ExpressionBuiltinSingleArgBase,
-    ExpressionChildrenHavingBase
+    ExpressionChildrenHavingBase,
 )
 
 
@@ -32,15 +32,12 @@ class ExpressionBuiltinNext1(ExpressionBuiltinSingleArgBase):
 
     def __init__(self, value, source_ref):
         ExpressionBuiltinSingleArgBase.__init__(
-            self,
-            value      = value,
-            source_ref = source_ref
+            self, value=value, source_ref=source_ref
         )
 
     def computeExpression(self, trace_collection):
         return self.getValue().computeExpressionNext1(
-            next_node        = self,
-            trace_collection = trace_collection
+            next_node=self, trace_collection=trace_collection
         )
 
 
@@ -48,11 +45,7 @@ class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
     kind = "EXPRESSION_SPECIAL_UNPACK"
 
     def __init__(self, value, count, expected, starred, source_ref):
-        ExpressionBuiltinNext1.__init__(
-            self,
-            value      = value,
-            source_ref = source_ref
-        )
+        ExpressionBuiltinNext1.__init__(self, value=value, source_ref=source_ref)
 
         self.count = int(count)
 
@@ -80,25 +73,25 @@ class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
 class ExpressionBuiltinNext2(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_NEXT2"
 
-    named_children = (
-        "iterator",
-        "default"
-    )
+    named_children = ("iterator", "default")
+    getIterator = ExpressionChildrenHavingBase.childGetter("iterator")
+    getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
     def __init__(self, iterator, default, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values     = {
-                "iterator" : iterator,
-                "default"  : default,
-            },
-            source_ref = source_ref
+            values={"iterator": iterator, "default": default},
+            source_ref=source_ref,
         )
 
-    getIterator = ExpressionChildrenHavingBase.childGetter("iterator")
-    getDefault = ExpressionChildrenHavingBase.childGetter("default")
-
     def computeExpression(self, trace_collection):
         # TODO: The "iterator" should be investigated here, if it is iterable,
         # or if the default is raising.
+
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(self)
+
+        # Any exception may be raised.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
