@@ -29,7 +29,7 @@ Coroutines and generators live in their dedicated module and share base
 classes.
 """
 
-from nuitka import Options, Variables
+from nuitka import Variables
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import (
     ParameterSpec,
@@ -37,6 +37,7 @@ from nuitka.specs.ParameterSpecs import (
     matchCall,
 )
 from nuitka.tree.Extractions import updateVariableUsage
+from nuitka.tree.TreeHelpers import makeDictCreationOrConstant2
 
 from .Checkers import checkStatementsSequenceOrNone
 from .CodeObjectSpecs import CodeObjectSpec
@@ -51,7 +52,7 @@ from .IndicatorMixins import (
     EntryPointMixin,
     MarkUnoptimizedFunctionIndicatorMixin,
 )
-from .LocalsScopes import getLocalsDictHandle, setLocalsDictType
+from .LocalsScopes import getLocalsDictHandle
 from .NodeBases import (
     ClosureGiverNodeMixin,
     ClosureTakerMixin,
@@ -114,10 +115,10 @@ class ExpressionFunctionBodyBase(
         return True
 
     def getEntryPoint(self):
-        """ Entry point for code.
+        """Entry point for code.
 
-            Normally ourselves. Only outlines will refer to their parent which
-            technically owns them.
+        Normally ourselves. Only outlines will refer to their parent which
+        technically owns them.
 
         """
 
@@ -143,7 +144,7 @@ class ExpressionFunctionBodyBase(
 
     @staticmethod
     def isEarlyClosure():
-        """ Early closure taking means immediate binding of references.
+        """Early closure taking means immediate binding of references.
 
         Normally it's good to lookup name references immediately, but not for
         functions. In case of a function body it is not allowed to do that,
@@ -154,55 +155,62 @@ class ExpressionFunctionBodyBase(
 
         return False
 
-    def hasVariableName(self, variable_name):
-        return variable_name in self.providing or variable_name in self.temp_variables
+    def getLocalsScope(self):
+        return self.locals_scope
 
-    def getVariables(self):
-        return self.providing.values()
+    # TODO: Dubious function doing to distinct things, should be moved to users.
+    def hasVariableName(self, variable_name):
+        return (
+            self.locals_scope.hasProvidedVariable(variable_name)
+            or variable_name in self.temp_variables
+        )
+
+    def getProvidedVariables(self):
+        if self.locals_scope is not None:
+            return self.locals_scope.getProvidedVariables()
+        else:
+            return ()
 
     def getLocalVariables(self):
         return [
             variable
-            for variable in self.providing.values()
+            for variable in self.getProvidedVariables()
             if variable.isLocalVariable()
         ]
 
     def getLocalVariableNames(self):
         return [
             variable.getName()
-            for variable in self.providing.values()
+            for variable in self.getProvidedVariables()
             if variable.isLocalVariable()
         ]
 
     def getUserLocalVariables(self):
-        return tuple(
+        return [
             variable
-            for variable in self.providing.values()
+            for variable in self.getProvidedVariables()
             if variable.isLocalVariable() and not variable.isParameterVariable()
             if variable.getOwner() is self
-        )
+        ]
 
     def getOutlineLocalVariables(self):
+        result = []
+
         outlines = self.getTraceCollection().getOutlineFunctions()
 
         if outlines is None:
-            return ()
-
-        result = []
+            return result
 
         for outline in outlines:
             result.extend(outline.getUserLocalVariables())
 
-        return tuple(result)
+        return result
 
     def removeClosureVariable(self, variable):
         # Do not remove parameter variables of ours.
         assert not variable.isParameterVariable() or variable.getOwner() is not self
 
-        variable_name = variable.getName()
-
-        if variable_name in self.providing:
-            del self.providing[variable.getName()]
+        self.locals_scope.unregisterClosureVariable(variable)
 
         self.taken.remove(variable)
 
@@ -221,7 +229,8 @@ class ExpressionFunctionBodyBase(
                 new_variable.addTrace(variable_trace)
         new_variable.updateUsageState()
 
-        self.providing[variable.getName()] = new_variable
+        self.locals_scope.unregisterClosureVariable(variable)
+        self.locals_scope.registerProvidedVariable(new_variable)
 
         updateVariableUsage(
             provider=self, old_variable=variable, new_variable=new_variable
@@ -231,12 +240,10 @@ class ExpressionFunctionBodyBase(
         return variable in self.taken
 
     def removeUserVariable(self, variable):
-        assert variable in self.providing.values(), (self, self.providing, variable)
-
-        del self.providing[variable.getName()]
-
         assert not variable.isParameterVariable() or variable.getOwner() is not self
 
+        self.locals_scope.unregisterProvidedVariable(variable)
+
     def getVariableForAssignment(self, variable_name):
         # print("ASS func", self, variable_name)
 
@@ -258,7 +265,7 @@ class ExpressionFunctionBodyBase(
             # Remember that we need that closure variable for something, so
             # we don't create it again all the time.
             if not result.isModuleVariable():
-                self.registerProvidedVariable(result)
+                self.locals_scope.registerClosureVariable(result)
 
             entry_point = self.getEntryPoint()
 
@@ -292,14 +299,18 @@ class ExpressionFunctionBodyBase(
     def createProvidedVariable(self, variable_name):
         # print("createProvidedVariable", self, variable_name)
 
-        return Variables.LocalVariable(owner=self, variable_name=variable_name)
+        assert self.locals_scope, self
+
+        return self.locals_scope.getLocalVariable(
+            variable_name=variable_name, owner=self
+        )
 
     def addNonlocalsDeclaration(self, names, user_provided, source_ref):
-        """ Add a nonlocal declared name.
+        """Add a nonlocal declared name.
 
-            This happens during tree building, and is a Python3 only
-            feature. We remember the names for later use through the
-            function @consumeNonlocalDeclarations
+        This happens during tree building, and is a Python3 only
+        feature. We remember the names for later use through the
+        function @consumeNonlocalDeclarations
         """
         if self.non_local_declarations is None:
             self.non_local_declarations = []
@@ -307,11 +318,11 @@ class ExpressionFunctionBodyBase(
         self.non_local_declarations.append((names, user_provided, source_ref))
 
     def consumeNonlocalDeclarations(self):
-        """ Return the nonlocal declared names for this function.
+        """Return the nonlocal declared names for this function.
 
-            There may not be any, which is why we assigned it to
-            None originally and now check and return empty tuple
-            in that case.
+        There may not be any, which is why we assigned it to
+        None originally and now check and return empty tuple
+        in that case.
         """
 
         result = self.non_local_declarations or ()
@@ -322,7 +333,7 @@ class ExpressionFunctionBodyBase(
         return self.name
 
     def getFunctionQualname(self):
-        """ Function __qualname__ new in CPython3.3
+        """Function __qualname__ new in CPython3.3
 
         Should contain some kind of full name descriptions for the closure to
         recognize and will be used for outputs.
@@ -356,6 +367,15 @@ class ExpressionFunctionBodyBase(
         else:
             return self.getBody().mayRaiseException(exception_type)
 
+    def getFunctionInlineCost(self, values):
+        """Cost of inlining this function with given arguments
+
+        Returns: None or integer values, None means don't do it.
+        """
+
+        # For overload, pylint: disable=no-self-use,unused-argument
+        return None
+
 
 class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBase):
     def __init__(
@@ -377,17 +397,14 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
 
         provider.getParentModule().addFunction(self)
 
-        if python_version >= 300:
-            self.locals_dict_name = "locals_%s" % (self.getCodeName(),)
-
-            setLocalsDictType(self.locals_dict_name, "python3_function")
-        elif flags is not None and "has_exec" in flags:
-            self.locals_dict_name = "locals_%s" % (self.getCodeName(),)
-
-            setLocalsDictType(self.locals_dict_name, "python2_function_exec")
+        if flags is not None and "has_exec" in flags:
+            locals_kind = "python2_function_exec"
         else:
-            # TODO: There should be a locals scope for non-dict/mapping too.
-            self.locals_dict_name = None
+            locals_kind = "python_function"
+
+        self.locals_scope = getLocalsDictHandle(
+            "locals_%s" % self.getCodeName(), locals_kind, self
+        )
 
         # Automatic parameter variable releases.
         self.auto_release = auto_release or None
@@ -399,17 +416,13 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
 
         return result
 
-    def getFunctionLocalsScope(self):
-        if self.locals_dict_name is None:
-            return None
-        else:
-            return getLocalsDictHandle(self.locals_dict_name)
-
     def getCodeObject(self):
         return self.code_object
 
     def computeFunctionRaw(self, trace_collection):
-        from nuitka.optimizations.TraceCollections import TraceCollectionFunction
+        from nuitka.optimizations.TraceCollections import (
+            TraceCollectionFunction,
+        )
 
         trace_collection = TraceCollectionFunction(
             parent=trace_collection, function_body=self
@@ -439,7 +452,7 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
                 self.setBody(result)
 
     def removeVariableReleases(self, variable):
-        assert variable in self.providing.values(), (self, self.providing, variable)
+        assert variable in self.locals_scope.providing.values(), (self, variable)
 
         if self.auto_release is None:
             self.auto_release = set()
@@ -447,35 +460,31 @@ class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBa
         self.auto_release.add(variable)
 
     def getParameterVariablesWithManualRelease(self):
-        """ Return the list of parameter variables that have release statements.
+        """Return the list of parameter variables that have release statements.
 
-            These are for consideration if these can be dropped, and if so, they
-            are releases automatically by function code.
+        These are for consideration if these can be dropped, and if so, they
+        are releases automatically by function code.
         """
         return tuple(
             variable
-            for variable in self.providing.values()
+            for variable in self.locals_scope.getProvidedVariables()
             if not self.auto_release or variable not in self.auto_release
             if variable.isParameterVariable()
             if variable.getOwner() is self
         )
 
     def isAutoReleaseVariable(self, variable):
-        """ Is this variable to be automatically released.
-
-        """
+        """Is this variable to be automatically released."""
         return self.auto_release is not None and variable in self.auto_release
 
     def getFunctionVariablesWithAutoReleases(self):
-        """ Return the list of function variables that should be released at exit.
-
-        """
+        """Return the list of function variables that should be released at exit."""
         if self.auto_release is None:
             return ()
 
         return tuple(
             variable
-            for variable in self.providing.values()
+            for variable in self.locals_scope.getProvidedVariables()
             if variable in self.auto_release
         )
 
@@ -535,7 +544,7 @@ class ExpressionFunctionBody(
         self.parameters.setOwner(self)
 
         for variable in self.parameters.getAllVariables():
-            self.registerProvidedVariable(variable)
+            self.locals_scope.registerProvidedVariable(variable)
 
     def getDetails(self):
         return {
@@ -599,6 +608,10 @@ class ExpressionFunctionBody(
             **other_args
         )
 
+    @staticmethod
+    def isExpressionFunctionBody():
+        return True
+
     def getParent(self):
         assert False
 
@@ -635,7 +648,7 @@ class ExpressionFunctionBody(
 
     def isCompileTimeConstant(self):
         # TODO: It's actually pretty much compile time accessible maybe.
-        return None
+        return False
 
     def mayHaveSideEffects(self):
         # The function definition has no side effects, calculating the defaults
@@ -654,6 +667,13 @@ class ExpressionFunctionBody(
         return self.return_exception
 
 
+class ExpressionFunctionPureBody(ExpressionFunctionBody):
+    kind = "EXPRESSION_FUNCTION_PURE_BODY"
+
+    def getFunctionInlineCost(self, values):
+        return 0
+
+
 def convertNoneConstantOrEmptyDictToNone(node):
     if node is None:
         return None
@@ -720,7 +740,7 @@ class ExpressionFunctionCreation(
             self.getFunctionRef().getFunctionBody().getClosureVariables()
         ):
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
             self.variable_closure_traces.append((closure_variable, trace))
 
@@ -795,8 +815,18 @@ class ExpressionFunctionCreation(
             if None in values:
                 return call_node, None, None
 
+            # TODO: This is probably something that the matchCall ought to do
+            # for us, but that will need cleanups. Also these functions and
+            # nodes ought to work with # ordered dictionaries maybe.
+            if call_spec.getStarDictArgumentName():
+                values[-1] = makeDictCreationOrConstant2(
+                    keys=[value[0] for value in values[-1]],
+                    values=[value[1] for value in values[-1]],
+                    source_ref=call_node.source_ref,
+                )
+
             result = ExpressionFunctionCall(
-                function=self, values=values, source_ref=call_node.getSourceReference()
+                function=self, values=values, source_ref=call_node.source_ref
             )
 
             return (
@@ -825,41 +855,6 @@ error"""
                 % self.getName(),
             )
 
-    def getCallCost(self, values):
-        # TODO: Ought to use values. If they are all constant, how about we
-        # assume no cost, pylint: disable=unused-argument
-
-        function_body = self.getFunctionRef().getFunctionBody()
-
-        if function_body.isExpressionClassBody():
-            if function_body.getBody().getStatements()[0].isStatementReturn():
-                return 0
-
-            return None
-
-        if True or not Options.isExperimental("function_inlining"):
-            return None
-
-        if function_body.isExpressionGeneratorObjectBody():
-            # TODO: That's not even allowed, is it?
-            assert False
-
-            return None
-
-        # TODO: Lying for the demo, this is too limiting, but needs frames to
-        # be allowed twice in a context.
-        if function_body.mayRaiseException(BaseException):
-            return 60
-
-        return 20
-
-    def createOutlineFromCall(self, provider, values):
-        from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
-
-        return convertFunctionCallToOutline(
-            provider=provider, function_ref=self.getFunctionRef(), values=values
-        )
-
     def getClosureVariableVersions(self):
         return self.variable_closure_traces
 
@@ -932,11 +927,11 @@ class ExpressionFunctionRef(ExpressionBase):
 
 
 class ExpressionFunctionCall(ExpressionChildrenHavingBase):
-    """ Shared function call.
+    """Shared function call.
 
-        This is for calling created function bodies with multiple users. Not
-        clear if such a thing should exist. But what this will do is to have
-        respect for the fact that there are multiple such calls.
+    This is for calling created function bodies with multiple users. Not
+    clear if such a thing should exist. But what this will do is to have
+    respect for the fact that there are multiple such calls.
     """
 
     kind = "EXPRESSION_FUNCTION_CALL"
@@ -957,34 +952,40 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
         self.variable_closure_traces = None
 
     def computeExpression(self, trace_collection):
-        function = self.getFunction()
-
-        values = self.getArgumentValues()
-
-        # TODO: This needs some design.
-        cost = function.getCallCost(values)
-
+        function = self.subnode_function
         function_body = function.getFunctionRef().getFunctionBody()
 
         if function_body.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
 
+        values = self.subnode_values
+
+        # Ask for function for its cost.
+        cost = function_body.getFunctionInlineCost(values)
+
         if cost is not None and cost < 50:
-            result = function.createOutlineFromCall(
-                provider=self.getParentVariableProvider(), values=values
+            from nuitka.optimizations.FunctionInlining import (
+                convertFunctionCallToOutline,
+            )
+
+            result = convertFunctionCallToOutline(
+                provider=self.getParentVariableProvider(),
+                function_body=function_body,
+                values=values,
+                call_source_ref=self.source_ref,
             )
 
             return (
                 result,
                 "new_statements",
-                "Function call to '%s' in-lined." % function_body.getCodeName(),
+                lambda: "Function call to '%s' in-lined." % function_body.getCodeName(),
             )
 
         self.variable_closure_traces = []
 
         for closure_variable in function_body.getClosureVariables():
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
             self.variable_closure_traces.append((closure_variable, trace))
 
@@ -1012,8 +1013,15 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
         return self.variable_closure_traces
 
 
-# Needed for Python3.3 and higher
 class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
+    """Node for value __qualname__ of class.
+
+    Notes:
+        This is for Python 3.4 and higher only, where classes calculate the __qualname__
+        value at runtime, then it's determined dynamically, while 3.3 set it more
+        statically, and Python2 didn't have this feature at all.
+    """
+
     kind = "EXPRESSION_FUNCTION_QUALNAME_REF"
 
     __slots__ = ("function_body",)
@@ -1038,3 +1046,6 @@ class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
             "Executed '__qualname__' resolution to '%s'."
             % self.function_body.getFunctionQualname(),
         )
+
+    def getCompileTimeConstant(self):
+        return self.function_body.getFunctionQualname()
