@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -41,13 +41,11 @@ from .ModuleAttributeNodes import (
     ExpressionModuleAttributeSpecRef,
 )
 from .NodeMakingHelpers import makeRaiseExceptionReplacementExpression
-from .shapes.StandardShapes import ShapeUnknown
+from .shapes.StandardShapes import tshape_unknown
 
 
 class ExpressionVariableNameRef(ExpressionBase):
-    """ These are used before the actual variable object is known from VariableClosure.
-
-    """
+    """These are used before the actual variable object is known from VariableClosure."""
 
     kind = "EXPRESSION_VARIABLE_NAME_REF"
 
@@ -85,11 +83,11 @@ class ExpressionVariableNameRef(ExpressionBase):
 
 
 class ExpressionVariableLocalNameRef(ExpressionVariableNameRef):
-    """ These are used before the actual variable object is known from VariableClosure.
+    """These are used before the actual variable object is known from VariableClosure.
 
-        The special thing about this as opposed to ExpressionVariableNameRef is that
-        these must remain local names and cannot fallback to outside scopes. This is
-        used for __annotations__.
+    The special thing about this as opposed to ExpressionVariableNameRef is that
+    these must remain local names and cannot fallback to outside scopes. This is
+    used for "__annotations__".
 
     """
 
@@ -127,7 +125,7 @@ class ExpressionVariableRefBase(ExpressionBase):
 
     def getTypeShape(self):
         if self.variable_trace is None:
-            return ShapeUnknown
+            return tshape_unknown
         else:
             return self.variable_trace.getTypeShape()
 
@@ -242,6 +240,24 @@ Subscript look-up to dictionary lowered to dictionary look-up."""
 
         return lookup_node, tags, message
 
+    def _applyReplacement(self, trace_collection, replacement):
+        trace_collection.signalChange(
+            "new_expression",
+            self.source_ref,
+            "Value propagated for '%s' from '%s'."
+            % (self.variable.getName(), replacement.getSourceReference().getAsString()),
+        )
+
+        # Special case for in-place assignments.
+        if self.parent.isExpressionOperationInplace():
+            statement = self.parent.parent
+
+            if statement.isStatementAssignmentVariable():
+                statement.unmarkAsInplaceSuspect()
+
+        # Need to compute the replacement still.
+        return replacement.computeExpressionRaw(trace_collection)
+
 
 _hard_names = ("dir", "eval", "exec", "execfile", "locals", "vars", "super")
 
@@ -258,6 +274,10 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
             self, variable=variable, source_ref=source_ref
         )
 
+    @staticmethod
+    def isExpressionVariableRef():
+        return True
+
     def getDetails(self):
         return {"variable": self.variable}
 
@@ -299,17 +319,8 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
         )
 
         replacement = self.variable_trace.getReplacementNode(self)
-
         if replacement is not None:
-            trace_collection.signalChange(
-                "new_expression",
-                self.source_ref,
-                "Value propagated for '%s' from '%s'."
-                % (variable.getName(), replacement.getSourceReference().getAsString()),
-            )
-
-            # Need to compute the replacement still.
-            return replacement.computeExpressionRaw(trace_collection)
+            return self._applyReplacement(trace_collection, replacement)
 
         if not self.variable_trace.mustHaveValue():
             # TODO: This could be way more specific surely.
@@ -319,7 +330,7 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
             variable_name = self.variable.getName()
 
             if variable_name in Builtins.builtin_exception_names:
-                if not self.variable.getOwner().getModuleDictScope().isEscaped():
+                if not self.variable.getOwner().getLocalsScope().isEscaped():
                     from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
 
                     new_node = ExpressionBuiltinExceptionRef(
@@ -342,12 +353,14 @@ Module variable '%s' found to be built-in exception reference.""" % (
             elif variable_name in Builtins.builtin_names:
                 if (
                     variable_name in _hard_names
-                    or not self.variable.getOwner().getModuleDictScope().isEscaped()
+                    or not self.variable.getOwner().getLocalsScope().isEscaped()
                 ):
                     from .BuiltinRefNodes import makeExpressionBuiltinRef
 
                     new_node = makeExpressionBuiltinRef(
-                        builtin_name=variable_name, source_ref=self.getSourceReference()
+                        builtin_name=variable_name,
+                        locals_scope=self.getFunctionsLocalsScope(),
+                        source_ref=self.getSourceReference(),
                     )
 
                     change_tags = "new_builtin_ref"
@@ -413,8 +426,7 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
             result = makeRaiseExceptionReplacementExpression(
                 expression=self,
                 exception_type="UnboundLocalError",
-                exception_value="""\
-local variable '%s' referenced before assignment"""
+                exception_value="""local variable '%s' referenced before assignment"""
                 % variable_name,
             )
 
@@ -438,7 +450,7 @@ local variable '%s' referenced before assignment"""
             and self.variable.isModuleVariable()
         ):
             # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage(self.getParentVariableProvider())
+            trace_collection.onLocalsUsage(locals_scope=self.getFunctionsLocalsScope())
 
         return call_node, None, None
 
@@ -464,6 +476,35 @@ local variable '%s' referenced before assignment"""
             or not self.variable_trace.getTypeShape().hasShapeSlotBool()
         )
 
+    def getFunctionsLocalsScope(self):
+        return self.getParentVariableProvider().getLocalsScope()
+
+
+class ExpressionVariableOrBuiltinRef(ExpressionVariableRef):
+    kind = "EXPRESSION_VARIABLE_OR_BUILTIN_REF"
+
+    __slots__ = ("locals_scope",)
+
+    def __init__(self, variable, locals_scope, source_ref):
+        ExpressionVariableRef.__init__(self, variable=variable, source_ref=source_ref)
+
+        self.locals_scope = locals_scope
+
+    def getDetails(self):
+        return {"variable": self.variable, "locals_scope": self.locals_scope}
+
+    def getFunctionsLocalsScope(self):
+        return self.locals_scope
+
+
+def makeExpressionVariableRef(variable, locals_scope, source_ref):
+    if variable.getName() in _hard_names:
+        return ExpressionVariableOrBuiltinRef(
+            variable=variable, locals_scope=locals_scope, source_ref=source_ref
+        )
+    else:
+        return ExpressionVariableRef(variable=variable, source_ref=source_ref)
+
 
 class ExpressionTempVariableRef(ExpressionVariableRefBase):
     kind = "EXPRESSION_TEMP_VARIABLE_REF"
@@ -504,13 +545,8 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
         )
 
         replacement = self.variable_trace.getReplacementNode(self)
-
         if replacement is not None:
-            return (
-                replacement,
-                "new_expression",
-                "Value propagated for temp '%s'." % self.variable.getName(),
-            )
+            return self._applyReplacement(trace_collection, replacement)
 
         self.variable_trace.addUsage()
 
@@ -519,7 +555,7 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
 
     def computeExpressionNext1(self, next_node, trace_collection):
         if self.variable_trace.isAssignTrace():
-            value = self.variable_trace.getAssignNode().getAssignSource()
+            value = self.variable_trace.getAssignNode().subnode_source
 
             # TODO: Add iteration handles to trace collections instead.
             current_index = trace_collection.getIteratorNextCount(value)
@@ -571,10 +607,8 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
 
     def mayRaiseExceptionImportName(self, exception_type, import_name):
         if self.variable_trace is not None and self.variable_trace.isAssignTrace():
-            return (
-                self.variable_trace.getAssignNode()
-                .getAssignSource()
-                .mayRaiseExceptionImportName(exception_type, import_name)
+            return self.variable_trace.getAssignNode().subnode_source.mayRaiseExceptionImportName(
+                exception_type, import_name
             )
 
         else:
