@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,47 +27,105 @@ The base class in PluginBase will serve as documentation of available.
 
 """
 
-from __future__ import print_function
-
 import os
 import pkgutil
+import shutil
 import sys
-from logging import info
+from optparse import OptionGroup
 
+import nuitka.plugins.commercial
 import nuitka.plugins.standard
 from nuitka import Options
 from nuitka.__past__ import basestring  # pylint: disable=I0021,redefined-builtin
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
+from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
 from nuitka.ModuleRegistry import addUsedModule
-from nuitka.PythonVersions import python_version
+from nuitka.Tracing import printLine
+from nuitka.utils.FileOperations import makePath
+from nuitka.utils.Importing import importFileAsModule
 from nuitka.utils.ModuleNames import ModuleName
 
 from .PluginBase import NuitkaPluginBase, post_modules, pre_modules
 
-active_plugin_list = []
+active_plugins = OrderedDict()
 plugin_name2plugin_classes = {}
+plugin_options = {}
+plugin_values = {}
+user_plugins = OrderedSet()
 
 
-def loadStandardPlugins():
-    """ Load plugin files located in 'standard' folder.
+def _addActivePlugin(plugin_class, args, force=False):
+    plugin_name = plugin_class.plugin_name
+
+    # No duplicates please.
+    if not force:
+        assert plugin_name not in active_plugins.keys(), (
+            plugin_name,
+            active_plugins[plugin_name],
+        )
+
+    if args:
+        plugin_args = getPluginOptions(plugin_name)
+    else:
+        plugin_args = {}
+
+    plugin_instance = plugin_class(**plugin_args)
+    assert isinstance(plugin_instance, NuitkaPluginBase), plugin_instance
+
+    active_plugins[plugin_name] = plugin_instance
+
+
+def getActivePlugins():
+    """Return list of active plugins.
 
-    Notes:
-        Scan through the 'standard' sub-folder of the folder where this script
-        resides. Import each valid Python script and process it as a plugin.
     Returns:
-        None
+        list of plugins
+
     """
 
-    # Complex stuff, pylint: disable=too-many-branches
+    return active_plugins.values()
 
-    for loader, name, is_pkg in pkgutil.iter_modules(nuitka.plugins.standard.__path__):
+
+def hasActivePlugin(plugin_name):
+    """Decide if a plugin is active.
+
+    Args:
+        plugin_name - name of the plugin
+
+    Notes:
+        Detectors do not count as an active plugin and ignored.
+
+    Returns:
+        bool - plugin is loaded
+
+    """
+    if plugin_name not in active_plugins:
+        return False
+
+    # Detectors do not count.
+    plugin_instance = active_plugins.get(plugin_name)
+    return not hasattr(plugin_instance, "detector_for")
+
+
+def getPluginClass(plugin_name):
+    # First, load plugin classes, to know what we are talking about.
+    loadPlugins()
+
+    if plugin_name not in plugin_name2plugin_classes:
+        sys.exit("Error, unknown plug-in '%s' referenced." % plugin_name)
+
+    return plugin_name2plugin_classes[plugin_name][0]
+
+
+def _loadPluginClassesFromPath(scan_path):
+    for loader, name, is_pkg in pkgutil.iter_modules(scan_path):
         if is_pkg:
             continue
 
         module_loader = loader.find_module(name)
 
-        # Ignore bytecode left overs.
+        # Ignore bytecode only left overs.
         try:
             if module_loader.get_filename().endswith(".pyc"):
                 continue
@@ -78,50 +136,49 @@ def loadStandardPlugins():
 
         plugin_module = module_loader.load_module(name)
 
-        plugin_objects = [None, None]  # plugin and optional detector
+        plugin_classes = set(
+            obj
+            for obj in plugin_module.__dict__.values()
+            if isObjectAUserPluginBaseClass(obj)
+        )
 
-        for key in dir(plugin_module):  # scan for plugin classes
-            obj = getattr(plugin_module, key)
-            if not isObjectAUserPluginBaseClass(obj):
-                continue
+        detectors = [
+            plugin_class
+            for plugin_class in plugin_classes
+            if hasattr(plugin_class, "detector_for")
+        ]
 
-            always_enable = obj.isAlwaysEnabled()
-            if not hasattr(obj, "isRelevant"):
-                is_relevant = None
-            else:
-                try:
-                    is_relevant = obj.isRelevant()
-                except AttributeError:  # will only happen with --plugin-list
-                    is_relevant = True
+        for detector in detectors:
+            plugin_class = detector.detector_for
+            assert detector.plugin_name is None, detector
+            detector.plugin_name = plugin_class.plugin_name
 
-            if always_enable:  # should this always be enabled?
-                if is_relevant in (True, None):
-                    plugin_objects[0] = obj
-                    plugin_objects[1] = None  # detector makes no sense!
-                else:
-                    plugin_objects = None
-                break
+            assert plugin_class in plugin_classes
 
-            # this is an optional plugin
-            if is_relevant is not None:  # must be the detector!
-                plugin_objects[1] = obj
-            else:
-                plugin_objects[0] = obj
+            plugin_classes.remove(detector)
+            plugin_classes.remove(plugin_class)
 
-        if plugin_objects is None:  # skip enabling irrelevant plugins
-            continue
-
-        plugin_objects = tuple(plugin_objects)
-        plugin = plugin_objects[0]
-        if plugin is None:
-            sys.exit("Plugin '%s' has no standard class." % plugin_module)
-
-        if always_enable:  # mandatory, relevant plugin: enable it
-            active_plugin_list.append(
-                (plugin or plugin)()  # TODO: pylint: disable=I0021,not-callable
+            plugin_name2plugin_classes[plugin_class.plugin_name] = (
+                plugin_class,
+                detector,
             )
-        else:  # optional plugin: make it searchable by plugin_name
-            plugin_name2plugin_classes[plugin.plugin_name] = plugin_objects
+
+        for plugin_class in plugin_classes:
+            plugin_name2plugin_classes[plugin_class.plugin_name] = plugin_class, None
+
+
+def loadStandardPluginClasses():
+    """Load plugin files located in 'standard' folder.
+
+    Notes:
+        Scan through the 'standard' and 'commercial' sub-folder of the folder
+        where this script resides. Import each valid Python module (but not
+        packages) and process it as a plugin.
+    Returns:
+        None
+    """
+    _loadPluginClassesFromPath(nuitka.plugins.standard.__path__)
+    _loadPluginClassesFromPath(nuitka.plugins.commercial.__path__)
 
 
 class Plugins(object):
@@ -129,15 +186,11 @@ class Plugins(object):
     def isPluginActive(plugin_name):
         """ Is a plugin activated. """
 
-        for plugin in active_plugin_list:
-            if plugin.plugin_name == plugin_name:
-                return True
-
-        return False
+        return plugin_name in active_plugins
 
     @staticmethod
     def considerImplicitImports(module, signal_change):
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             plugin.considerImplicitImports(module, signal_change)
 
         # Post load code may have been created, if so indicate it's used.
@@ -151,9 +204,8 @@ class Plugins(object):
 
     @staticmethod
     def onStandaloneDistributionFinished(dist_dir):
-        """ Let plugins postprocess the distribution folder if standalone
-        """
-        for plugin in active_plugin_list:
+        """Let plugins postprocess the distribution folder if standalone"""
+        for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(dist_dir)
 
         return None
@@ -162,19 +214,31 @@ class Plugins(object):
     def considerExtraDlls(dist_dir, module):
         result = []
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             for extra_dll in plugin.considerExtraDlls(dist_dir, module):
-                if not os.path.isfile(extra_dll[0]):
-                    sys.exit(
-                        "Error, attempting to copy plugin determined filename %r for module %r that is not a file."
-                        % (extra_dll[0], module.getFullName())
+                # Backward compatibility with plugins not yet migrated to getExtraDlls usage.
+                if len(extra_dll) == 3:
+                    extra_dll = makeDllEntryPointOld(
+                        source_path=extra_dll[0],
+                        dest_path=extra_dll[1],
+                        package_name=extra_dll[2],
                     )
 
-                if not os.path.isfile(extra_dll[1]):
-                    sys.exit(
-                        "Error, copied filename %r for module %r that is not a file."
-                        % (extra_dll[1], module.getFullName())
-                    )
+                    if not os.path.isfile(extra_dll.dest_path):
+                        sys.exit(
+                            "Error, copied filename %r for module %r that is not a file."
+                            % (extra_dll.dest_path, module.getFullName())
+                        )
+                else:
+                    if not os.path.isfile(extra_dll.source_path):
+                        sys.exit(
+                            "Error, attempting to copy plugin determined filename %r for module %r that is not a file."
+                            % (extra_dll.source_path, module.getFullName())
+                        )
+
+                    makePath(os.path.dirname(extra_dll.dest_path))
+
+                    shutil.copyfile(extra_dll.source_path, extra_dll.dest_path)
 
                 result.append(extra_dll)
 
@@ -182,7 +246,7 @@ class Plugins(object):
 
     @staticmethod
     def removeDllDependencies(dll_filename, dll_filenames):
-        """ Create list of removable shared libraries by scanning through the plugins.
+        """Create list of removable shared libraries by scanning through the plugins.
 
         Args:
             dll_filename: shared library filename
@@ -194,7 +258,7 @@ class Plugins(object):
 
         result = []
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             for removed_dll in plugin.removeDllDependencies(
                 dll_filename, dll_filenames
             ):
@@ -204,7 +268,7 @@ class Plugins(object):
 
     @staticmethod
     def considerDataFiles(module):
-        """ For a given module, ask plugins for any needed data files it may require.
+        """For a given module, ask plugins for any needed data files it may require.
 
         Args:
             module: module object
@@ -212,13 +276,14 @@ class Plugins(object):
             Data file description pairs, either (source, dest) or (func, dest)
             where the func will be called to create the content dynamically.
         """
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             for value in plugin.considerDataFiles(module):
-                yield value
+                if value:
+                    yield plugin.plugin_name, value
 
     @staticmethod
     def onModuleDiscovered(module):
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             plugin.onModuleDiscovered(module)
 
     @staticmethod
@@ -226,7 +291,7 @@ class Plugins(object):
         assert type(module_name) is ModuleName
         assert type(source_code) is str
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             source_code = plugin.onModuleSourceCode(module_name, source_code)
             assert type(source_code) is str
 
@@ -237,7 +302,7 @@ class Plugins(object):
         assert type(module_name) is ModuleName
         assert type(source_code) is str
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             source_code = plugin.onFrozenModuleSourceCode(
                 module_name, is_package, source_code
             )
@@ -250,7 +315,7 @@ class Plugins(object):
         assert type(module_name) is ModuleName
         assert bytecode.__class__.__name__ == "code"
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             bytecode = plugin.onFrozenModuleBytecode(module_name, is_package, bytecode)
             assert bytecode.__class__.__name__ == "code"
 
@@ -260,7 +325,7 @@ class Plugins(object):
     def onModuleEncounter(module_filename, module_name, module_kind):
         result = False
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             must_recurse = plugin.onModuleEncounter(
                 module_filename, module_name, module_kind
             )
@@ -271,7 +336,7 @@ class Plugins(object):
 
     @staticmethod
     def considerFailedImportReferrals(module_name):
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             new_module_name = plugin.considerFailedImportReferrals(module_name)
 
             if new_module_name is not None:
@@ -279,29 +344,9 @@ class Plugins(object):
 
         return None
 
-    @staticmethod
-    def suppressBuiltinImportWarning(module, source_ref):
-        """ Let plugins decide whether to suppress import warnings for builtin modules.
-
-        Notes:
-            Return will be True if at least one plugin returns other than False or None,
-            else False.
-
-        Args:
-            module: the module object
-            source_ref: source reference object
-        Returns:
-            True or False
-        """
-        for plugin in active_plugin_list:
-            if plugin.suppressBuiltinImportWarning(module, source_ref):
-                return True
-
-        return False
-
     @staticmethod
     def suppressUnknownImportWarning(importing, module_name):
-        """ Let plugins decide whether to suppress import warnings for an unknown module.
+        """Let plugins decide whether to suppress import warnings for an unknown module.
 
         Notes:
             If all plugins return False or None, the return will be False, else True.
@@ -318,7 +363,7 @@ class Plugins(object):
 
         source_ref = importing.getSourceReference()
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             if plugin.suppressUnknownImportWarning(
                 importing_module, module_name, source_ref
             ):
@@ -328,7 +373,7 @@ class Plugins(object):
 
     @staticmethod
     def decideCompilation(module_name, source_ref):
-        """ Let plugins decide whether to compile a module.
+        """Let plugins decide whether to compile a module.
 
         Notes:
             The decision is made by the first plugin not returning None.
@@ -336,7 +381,7 @@ class Plugins(object):
         Returns:
             "compiled" (default) or "bytecode".
         """
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             value = plugin.decideCompilation(module_name, source_ref)
 
             if value is not None:
@@ -347,7 +392,7 @@ class Plugins(object):
 
     @staticmethod
     def getPreprocessorSymbols():
-        """ Let plugins provide C defines to be used in compilation.
+        """Let plugins provide C defines to be used in compilation.
 
         Notes:
             The plugins can each contribute, but are hopefully using
@@ -359,7 +404,7 @@ class Plugins(object):
         """
         result = OrderedDict()
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             value = plugin.getPreprocessorSymbols()
 
             if value is not None:
@@ -376,7 +421,7 @@ class Plugins(object):
     def getExtraCodeFiles():
         result = OrderedDict()
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             value = plugin.getExtraCodeFiles()
 
             if value is not None:
@@ -394,7 +439,7 @@ class Plugins(object):
     def getExtraLinkLibraries():
         result = OrderedSet()
 
-        for plugin in active_plugin_list:
+        for plugin in getActivePlugins():
             value = plugin.getExtraLinkLibraries()
 
             if value is not None:
@@ -408,10 +453,13 @@ class Plugins(object):
 
 
 def listPlugins():
-    """ Print available standard plugins.
-    """
-    print("The following optional standard plugins are available in Nuitka".center(80))
-    print("-" * 80)
+    """Print available standard plugins."""
+
+    loadPlugins()
+
+    printLine("The following plugins are available in Nuitka".center(80))
+    printLine("-" * 80)
+
     plist = []
     name_len = 0
     for plugin_name in sorted(plugin_name2plugin_classes):
@@ -422,78 +470,21 @@ def listPlugins():
             plist.append((plugin_name, ""))
         name_len = max(len(plugin_name) + 1, name_len)
     for line in plist:
-        print(" " + line[0].ljust(name_len), line[1])
+        printLine(" " + line[0].ljust(name_len), line[1])
 
     sys.exit(0)
 
 
 def isObjectAUserPluginBaseClass(obj):
-    """ Verify that a user plugin inherits from UserPluginBase.
-    """
+    """Verify that a user plugin inherits from UserPluginBase."""
     try:
         return obj is not NuitkaPluginBase and issubclass(obj, NuitkaPluginBase)
     except TypeError:
         return False
 
 
-def importFilePy3NewWay(filename):
-    """ Import a file for Python versions 3.5+.
-    """
-    import importlib.util  # pylint: disable=I0021,import-error,no-name-in-module
-
-    spec = importlib.util.spec_from_file_location(filename, filename)
-    user_plugin_module = importlib.util.module_from_spec(spec)
-    spec.loader.exec_module(user_plugin_module)
-    return user_plugin_module
-
-
-def importFilePy3OldWay(filename):
-    """ Import a file for Python versions 3.x where x < 5.
-    """
-    from importlib.machinery import (  # pylint: disable=I0021,import-error,no-name-in-module
-        SourceFileLoader,
-    )
-
-    # pylint: disable=I0021,deprecated-method
-    return SourceFileLoader(filename, filename).load_module(filename)
-
-
-def importFilePy2(filename):
-    """ Import a file for Python version 2.
-    """
-    import imp
-
-    basename = os.path.splitext(filename)[0]
-
-    name = os.path.join(
-        "nuitka", os.path.relpath(basename, os.path.dirname(nuitka.__file__))
-    ).replace(os.sep, ".")
-
-    return imp.load_source(name, filename)
-
-
-def importFile(filename):
-    """ Import Python module given as a file name.
-
-    Notes:
-        Provides a Python version independent way to import any script files.
-
-    Args:
-        filename: complete path of a Python script
-
-    Returns:
-        Imported Python module defined in filename.
-    """
-    if python_version < 300:
-        return importFilePy2(filename)
-    elif python_version < 350:
-        return importFilePy3OldWay(filename)
-    else:
-        return importFilePy3NewWay(filename)
-
-
-def importUserPlugins():
-    """ Extract the filenames of user plugins and store them in list of active plugins.
+def loadUserPlugin(plugin_filename):
+    """Load of a user plugins and store them in list of active plugins.
 
     Notes:
         A plugin is accepted only if it has a non-empty variable plugin_name, which
@@ -502,38 +493,43 @@ def importUserPlugins():
     Returns:
         None
     """
-    for plugin_filename in Options.getUserPlugins():
-        plugin_filename = plugin_filename.split("=", 1)[0]
-        if not os.path.exists(plugin_filename):
-            sys.exit("Error, cannot find '%s'." % plugin_filename)
+    if not os.path.exists(plugin_filename):
+        sys.exit("Error, cannot find '%s'." % plugin_filename)
 
-        user_plugin_module = importFile(plugin_filename)
-        valid_file = False
-        for key in dir(user_plugin_module):
-            obj = getattr(user_plugin_module, key)
-            if not isObjectAUserPluginBaseClass(obj):
-                continue
+    user_plugin_module = importFileAsModule(plugin_filename)
 
-            plugin_name = getattr(obj, "plugin_name", None)
-            if plugin_name and plugin_name not in Options.getPluginsDisabled():
-                info("User plugin '%s' is being loaded." % plugin_name)
-                active_plugin_list.append(obj())
-                valid_file = True
-                break  # do not look for more in that module
+    valid_file = False
+    plugin_class = None
+    for key in dir(user_plugin_module):
+        obj = getattr(user_plugin_module, key)
+        if not isObjectAUserPluginBaseClass(obj):
+            continue
 
-        if valid_file is False:  # this is not a plugin file ...
-            sys.exit("Error, '%s' is not a plugin file." % plugin_filename)
+        plugin_name = getattr(obj, "plugin_name", None)
+        if plugin_name and plugin_name not in Options.getPluginsDisabled():
+            plugin_class = obj
+
+            valid_file = True
+            break  # do not look for more in that module
+
+    if not valid_file:  # this is not a plugin file ...
+        sys.exit("Error, '%s' is not a plugin file." % plugin_filename)
+
+    return plugin_class
 
 
-def initPlugins():
-    """ Initialize plugins
+_loaded_plugins = False
+
+
+def loadPlugins():
+    """Initialize plugin class
 
     Notes:
         Load user plugins provided as Python script file names, and standard
         plugins via their class attribute 'plugin_name'.
-        Several checks are made, see below.
-        The final result is 'active_plugin_list' which contains all enabled
-        plugins.
+
+        Several checks are made, see the loader functions.
+
         User plugins are enabled as a first step, because they themselves may
         enable standard plugins.
 
@@ -541,11 +537,32 @@ def initPlugins():
         None
     """
 
-    # load user plugins first to allow any preparative action
-    importUserPlugins()
+    # Singleton, called potentially multiple times, pylint: disable=global-statement
+    global _loaded_plugins
+    if not _loaded_plugins:
+        _loaded_plugins = True
 
-    # now enable standard plugins
-    loadStandardPlugins()
+        # now enable standard plugins
+        loadStandardPluginClasses()
+
+
+def activatePlugins():
+    """Activate selected plugin classes
+
+    Args:
+        None
+
+    Notes:
+        This creates actual plugin instances, before only class objects were
+        used.
+
+        User plugins are activated as a first step, because they themselves may
+        enable standard plugins.
+
+    Returns:
+        None
+    """
+    loadPlugins()
 
     # ensure plugin is known and not both, enabled and disabled
     for plugin_name in Options.getPluginsEnabled() + Options.getPluginsDisabled():
@@ -558,19 +575,111 @@ def initPlugins():
         ):
             sys.exit("Error, conflicting enable/disable of plug-in '%s'." % plugin_name)
 
-    for (
-        plugin_name,
-        (plugin_class, plugin_detector),
-    ) in plugin_name2plugin_classes.items():
+    for (plugin_name, (plugin_class, plugin_detector)) in sorted(
+        plugin_name2plugin_classes.items()
+    ):
         if plugin_name in Options.getPluginsEnabled():
-            active_plugin_list.append(plugin_class())
-        elif plugin_name not in Options.getPluginsDisabled():
-            if (
-                plugin_detector is not None
-                and Options.shallDetectMissingPlugins()
-                and plugin_detector.isRelevant()
-            ):
-                active_plugin_list.append(plugin_detector())
+            if plugin_class.isRelevant():
+                _addActivePlugin(plugin_class, args=True)
+            else:
+                plugin_class.warning(
+                    "Not relevant with this OS, or Nuitka arguments given, not activated."
+                )
+        elif plugin_name in Options.getPluginsDisabled():
+            pass
+        elif plugin_class.isAlwaysEnabled() and plugin_class.isRelevant():
+            _addActivePlugin(plugin_class, args=True)
+        elif (
+            plugin_detector is not None
+            and Options.shallDetectMissingPlugins()
+            and plugin_detector.isRelevant()
+        ):
+            _addActivePlugin(plugin_detector, args=False)
+
+    for plugin_class in user_plugins:
+        _addActivePlugin(plugin_class, args=True)
 
 
-initPlugins()
+def lateActivatePlugin(plugin_name, option_values):
+    """Activate plugin after the command line parsing, expects options to be set."""
+
+    values = getPluginClass(plugin_name).getPluginDefaultOptionValues()
+    values.update(option_values)
+    setPluginOptions(plugin_name, values)
+
+    _addActivePlugin(getPluginClass(plugin_name), args=True, force=True)
+
+
+def _addPluginCommandLineOptions(parser, plugin_class):
+    option_group = OptionGroup(parser, "Plugin %s" % plugin_class.plugin_name)
+    plugin_class.addPluginCommandLineOptions(option_group)
+
+    if option_group.option_list:
+        parser.add_option_group(option_group)
+        plugin_options[plugin_class.plugin_name] = option_group.option_list
+
+
+def addPluginCommandLineOptions(parser, plugin_names):
+    """Add option group for the plugin to the parser.
+
+    Notes:
+        This is exclusively for use in the commandline parsing. Not all
+        plugins have to have options. But this will add them to the
+        parser in a first pass, so they can be recognized in a second
+        pass with them included.
+
+    Returns:
+        None
+    """
+    for plugin_name in plugin_names:
+        plugin_class = getPluginClass(plugin_name)
+        _addPluginCommandLineOptions(parser, plugin_class)
+
+
+def addUserPluginCommandLineOptions(parser, filename):
+    plugin_class = loadUserPlugin(filename)
+    _addPluginCommandLineOptions(parser, plugin_class)
+
+    user_plugins.add(plugin_class)
+
+
+def setPluginOptions(plugin_name, values):
+    """Set the option values for the specified plugin.
+
+    Args:
+        plugin_name: plugin identifier
+        values: dictionary to be used for the plugin constructor
+    Notes:
+        Use this function, if you want to set the plugin values, without using
+        the actual command line parsing.
+
+        Normally the command line arguments are populating the dictionary for
+        the plugin, but this will be used if given, and command line parsing
+        is not done.
+    """
+    assert isinstance(values, dict), values
+    plugin_values[plugin_name] = values
+
+
+def getPluginOptions(plugin_name):
+    """Return the options values for the specified plugin.
+
+    Args:
+        plugin_name: plugin identifier
+    Returns:
+        dict with key, value of options given, potentially from default values.
+    """
+    result = plugin_values.get(plugin_name, {})
+
+    for option in plugin_options.get(plugin_name, {}):
+        option_name = option._long_opts[0]  # pylint: disable=protected-access
+
+        arg_value = getattr(Options.options, option.dest)
+
+        if "[REQUIRED]" in option.help:
+            if not arg_value:
+                sys.exit("Error, required plugin argument %r not given." % option_name)
+
+        result[option.dest] = arg_value
+
+    return result
