@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,7 +32,7 @@ from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
 from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
 from nuitka.nodes.BuiltinRefNodes import (
     ExpressionBuiltinAnonymousRef,
-    makeExpressionBuiltinRef,
+    makeExpressionBuiltinTypeRef,
 )
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinTuple
 from nuitka.nodes.CallNodes import makeExpressionCall
@@ -63,12 +63,18 @@ from nuitka.nodes.SubscriptNodes import (
     ExpressionSubscriptLookup,
     StatementAssignmentSubscript,
 )
-from nuitka.nodes.TypeNodes import ExpressionBuiltinIsinstance, ExpressionBuiltinType1
+from nuitka.nodes.TypeNodes import (
+    ExpressionBuiltinIsinstance,
+    ExpressionBuiltinType1,
+)
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
-from nuitka.PythonVersions import getComplexCallSequenceErrorTemplate, python_version
+from nuitka.PythonVersions import (
+    getComplexCallSequenceErrorTemplate,
+    python_version,
+)
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
 from .InternalModule import (
@@ -108,7 +114,7 @@ def orderArgs(*args):
 
 def _makeNameAttributeLookup(node, attribute_name="__name__"):
     return ExpressionAttributeLookup(
-        source=node, attribute_name=attribute_name, source_ref=internal_source_ref
+        expression=node, attribute_name=attribute_name, source_ref=internal_source_ref
     )
 
 
@@ -120,13 +126,13 @@ def getCallableNameDescBody():
     #
     # Note: The "called_type" is a temporary variable.
     #
-    # called_type = type( BuiltinFunctionType )
+    # called_type = type(BuiltinFunctionType)
     #
     # if ininstance( called, (FunctionType, MethodType, BuiltinFunctionType) ):
     #     return called.__name__
-    # elif python_version < 3 and isinstance( called, ClassType ):
+    # elif python_version < 3 and isinstance(called, ClassType):
     #     return called_type.__name__ + " constructor"
-    # elif python_version < 3 and isinstance( called, InstanceType ):
+    # elif python_version < 3 and isinstance(called, InstanceType):
     #     return called_type.__name__ + " instance"
     # else:
     #     return called_type.__name__ + " object"
@@ -168,6 +174,11 @@ def getCallableNameDescBody():
         )
     )
 
+    if python_version >= 390:
+        result.setBody(functions_case)
+
+        return result
+
     no_branch = StatementReturn(
         expression=makeBinaryOperationNode(
             operator="Add",
@@ -269,6 +280,7 @@ def getCallableNameDescBody():
                     instance=ExpressionVariableRef(
                         variable=called_variable, source_ref=internal_source_ref
                     ),
+                    # TODO: This ought to be a constant value.
                     classes=ExpressionMakeTuple(
                         elements=tuple(
                             ExpressionBuiltinAnonymousRef(
@@ -353,14 +365,14 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
         non_tuple_code = makeStatementConditional(
             condition=ExpressionConditionalOr(
                 left=ExpressionAttributeCheck(
-                    object_arg=ExpressionVariableRef(
+                    expression=ExpressionVariableRef(
                         variable=star_list_variable, source_ref=internal_source_ref
                     ),
                     attribute_name="__iter__",
                     source_ref=internal_source_ref,
                 ),
                 right=ExpressionAttributeCheck(
-                    object_arg=ExpressionVariableRef(
+                    expression=ExpressionVariableRef(
                         variable=star_list_variable, source_ref=internal_source_ref
                     ),
                     attribute_name="__getitem__",
@@ -410,7 +422,7 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
                 ),
                 source_ref=internal_source_ref,
             ),
-            right=makeExpressionBuiltinRef(
+            right=makeExpressionBuiltinTypeRef(
                 builtin_name="tuple", source_ref=internal_source_ref
             ),
             source_ref=internal_source_ref,
@@ -510,7 +522,29 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
     tmp_keys_variable = result.allocateTempVariable(temp_scope, "keys")
     tmp_key_variable = result.allocateTempVariable(temp_scope, "key")
 
-    loop_body = (
+    loop_body = []
+
+    if python_version >= 380:
+        loop_body.append(
+            makeStatementConditional(
+                condition=ExpressionComparisonIn(
+                    left=ExpressionTempVariableRef(
+                        variable=tmp_key_variable, source_ref=internal_source_ref
+                    ),
+                    right=ExpressionVariableRef(
+                        variable=tmp_dict_variable, source_ref=internal_source_ref
+                    ),
+                    source_ref=internal_source_ref,
+                ),
+                yes_branch=_makeRaiseDuplicationItem(
+                    called_variable=called_variable, tmp_key_variable=tmp_key_variable
+                ),
+                no_branch=None,
+                source_ref=internal_source_ref,
+            )
+        )
+
+    loop_body.append(
         StatementAssignmentSubscript(
             expression=ExpressionTempVariableRef(
                 variable=tmp_dict_variable, source_ref=internal_source_ref
@@ -519,7 +553,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
                 variable=tmp_key_variable, source_ref=internal_source_ref
             ),
             source=ExpressionSubscriptLookup(
-                subscribed=ExpressionVariableRef(
+                expression=ExpressionVariableRef(
                     variable=star_dict_variable, source_ref=internal_source_ref
                 ),
                 subscript=ExpressionTempVariableRef(
@@ -528,7 +562,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
                 source_ref=internal_source_ref,
             ),
             source_ref=internal_source_ref,
-        ),
+        )
     )
 
     mapping_case = makeStatementsSequenceFromStatements(
@@ -591,7 +625,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
                 ),
                 source_ref=internal_source_ref,
             ),
-            right=makeExpressionBuiltinRef(
+            right=makeExpressionBuiltinTypeRef(
                 builtin_name="dict", source_ref=internal_source_ref
             ),
             source_ref=internal_source_ref,
@@ -768,7 +802,7 @@ def _makeStarDictArgumentMergeToKwStatement(
                 variable=tmp_key_variable, source_ref=internal_source_ref
             ),
             source=ExpressionSubscriptLookup(
-                subscribed=ExpressionVariableRef(
+                expression=ExpressionVariableRef(
                     variable=star_dict_variable, source_ref=internal_source_ref
                 ),
                 subscript=ExpressionTempVariableRef(
@@ -831,7 +865,7 @@ def _makeStarDictArgumentMergeToKwStatement(
     tmp_item_variable = result.allocateTempVariable(temp_scope, "item")
     tmp_key_variable = result.allocateTempVariable(temp_scope, "key")
 
-    final += [
+    final += (
         StatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
@@ -841,13 +875,13 @@ def _makeStarDictArgumentMergeToKwStatement(
         StatementReleaseVariable(
             variable=tmp_key_variable, source_ref=internal_source_ref
         ),
-    ]
+    )
 
     dict_loop_body = (
         StatementAssignmentVariable(
             variable=tmp_key_variable,
             source=ExpressionSubscriptLookup(
-                subscribed=ExpressionTempVariableRef(
+                expression=ExpressionTempVariableRef(
                     variable=tmp_item_variable, source_ref=internal_source_ref
                 ),
                 subscript=makeConstantRefNode(
@@ -881,7 +915,7 @@ def _makeStarDictArgumentMergeToKwStatement(
                 variable=tmp_key_variable, source_ref=internal_source_ref
             ),
             source=ExpressionSubscriptLookup(
-                subscribed=ExpressionTempVariableRef(
+                expression=ExpressionTempVariableRef(
                     variable=tmp_item_variable, source_ref=internal_source_ref
                 ),
                 subscript=makeConstantRefNode(
@@ -945,7 +979,7 @@ def _makeStarDictArgumentMergeToKwStatement(
                 ),
                 source_ref=internal_source_ref,
             ),
-            right=makeExpressionBuiltinRef(
+            right=makeExpressionBuiltinTypeRef(
                 builtin_name="dict", source_ref=internal_source_ref
             ),
             source_ref=internal_source_ref,
@@ -968,16 +1002,16 @@ def getFunctionCallHelperStarList():
     # Note: Call in here is not the same, as it can go without checks directly
     # to PyObject_Call.
     #
-    # if not isinstance( star_arg_list, tuple ):
+    # if not isinstance(star_arg_list, tuple):
     #     try:
-    #         star_arg_list = tuple( star_arg_list )
+    #         star_arg_list = tuple(star_arg_list)
     #     except TypeError:
     #         raise TypeError, "%s argument after * must be a sequence, not %s" % (
-    #             get_callable_name_desc( function ),
-    #             type( star_arg_list ).__name__
+    #             get_callable_name_desc(function),
+    #             type(star_arg_list).__name__
     #         )
     #
-    # return called( *star_arg_list )
+    # return called(*star_arg_list)
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
@@ -999,7 +1033,7 @@ def getFunctionCallHelperStarList():
         variable_name="star_arg_list"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarListArgumentToTupleStatement(
             called_variable=called_variable, star_list_variable=star_arg_list_variable
         ),
@@ -1018,25 +1052,7 @@ def getFunctionCallHelperStarList():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1050,16 +1066,16 @@ def getFunctionCallHelperKeywordsStarList():
     # Note: Call in here is not the same, as it can go without checks directly
     # to PyObject_Call.
     #
-    # if not isinstance( star_arg_list, tuple ):
+    # if not isinstance(star_arg_list, tuple):
     #     try:
-    #         star_arg_list = tuple( star_arg_list )
+    #         star_arg_list = tuple(star_arg_list)
     #     except TypeError:
     #         raise TypeError, "%s argument after * must be a sequence, not %s" % (
-    #             get_callable_name_desc( function ),
-    #             type( star_arg_list ).__name__
+    #             get_callable_name_desc(function),
+    #             type(star_arg_list).__name__
     #         )
     #
-    # return called( *star_arg_list )
+    # return called(*star_arg_list)
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
@@ -1084,7 +1100,7 @@ def getFunctionCallHelperKeywordsStarList():
         variable_name="star_arg_list"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarListArgumentToTupleStatement(
             called_variable=called_variable, star_list_variable=star_arg_list_variable
         ),
@@ -1105,26 +1121,7 @@ def getFunctionCallHelperKeywordsStarList():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1138,16 +1135,16 @@ def getFunctionCallHelperPosStarList():
     # Note: Call in here is not the same, as it can go without checks directly
     # to PyObject_Call.
     #
-    # if not isinstance( star_arg_list, tuple ):
+    # if not isinstance(star_arg_list, tuple):
     #     try:
-    #         star_arg_list = tuple( star_arg_list )
+    #         star_arg_list = tuple(star_arg_list)
     #     except TypeError:
     #         raise TypeError, "%s argument after * must be a sequence, not %s" % (
-    #             get_callable_name_desc( function ),
-    #             type( star_arg_list ).__name__
+    #             get_callable_name_desc(function),
+    #             type(star_arg_list).__name__
     #         )
     #
-    # return called( *star_arg_list )
+    # return called(*star_arg_list)
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
@@ -1172,7 +1169,7 @@ def getFunctionCallHelperPosStarList():
         variable_name="star_arg_list"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarListArgumentToTupleStatement(
             called_variable=called_variable, star_list_variable=star_arg_list_variable
         ),
@@ -1198,28 +1195,7 @@ def getFunctionCallHelperPosStarList():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1232,16 +1208,16 @@ def getFunctionCallHelperPosKeywordsStarList():
     # Note: Call in here is not the same, as it can go without checks directly
     # to PyObject_Call.
     #
-    # if not isinstance( star_arg_list, tuple ):
+    # if not isinstance(star_arg_list, tuple):
     #     try:
-    #         star_arg_list = tuple( star_arg_list )
+    #         star_arg_list = tuple(star_arg_list)
     #     except TypeError:
     #         raise TypeError, "%s argument after * must be a sequence, not %s" % (
-    #             get_callable_name_desc( function ),
-    #             type( star_arg_list ).__name__
+    #             get_callable_name_desc(function),
+    #             type(star_arg_list).__name__
     #         )
     #
-    # return called( *star_arg_list )
+    # return called(*star_arg_list)
 
     # Only need to check if the star argument value is a sequence and then
     # convert to tuple.
@@ -1268,7 +1244,7 @@ def getFunctionCallHelperPosKeywordsStarList():
         variable_name="star_arg_list"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarListArgumentToTupleStatement(
             called_variable=called_variable, star_list_variable=star_arg_list_variable
         ),
@@ -1296,29 +1272,7 @@ def getFunctionCallHelperPosKeywordsStarList():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1336,7 +1290,7 @@ def getFunctionCallHelperStarDict():
     #         tmp_keys =  star_arg_dict.keys()
     #     except AttributeError:
     #         raise TypeError, ""%s argument after ** must be a mapping, not %s" % (
-    #             get_callable_name_desc( function ),
+    #             get_callable_name_desc(function),
     #             type(star_arg_dict).__name__
     #         )
     #
@@ -1376,7 +1330,7 @@ def getFunctionCallHelperStarDict():
         variable_name="star_arg_dict"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarDictArgumentToDictStatement(
             result=result,
             called_variable=called_variable,
@@ -1397,25 +1351,7 @@ def getFunctionCallHelperStarDict():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1476,7 +1412,7 @@ def getFunctionCallHelperPosStarDict():
         variable_name="star_arg_dict"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarDictArgumentToDictStatement(
             result=result,
             called_variable=called_variable,
@@ -1499,28 +1435,7 @@ def getFunctionCallHelperPosStarDict():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1540,7 +1455,7 @@ def getFunctionCallHelperKeywordsStarDict():
     #     except AttributeError:
     #         raise TypeError, ""%s argument after ** must be a mapping, not %s" % (
     #             get_callable_name_desc(function),
-    #             type( star_arg_dict ).__name__
+    #             type(star_arg_dict).__name__
     #         )
     #
     #     if keys:
@@ -1606,7 +1521,7 @@ def getFunctionCallHelperKeywordsStarDict():
         variable_name="star_arg_dict"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarDictArgumentMergeToKwStatement(
             result=result,
             called_variable=called_variable,
@@ -1628,26 +1543,7 @@ def getFunctionCallHelperKeywordsStarDict():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1735,7 +1631,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
         variable_name="star_arg_dict"
     )
 
-    statements = (
+    body = makeStatementsSequenceFromStatements(
         _makeStarDictArgumentMergeToKwStatement(
             result=result,
             called_variable=called_variable,
@@ -1759,29 +1655,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
-
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1874,28 +1748,9 @@ def getFunctionCallHelperStarListStarDict():
         )
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
+    body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -1967,31 +1822,9 @@ def getFunctionCallHelperPosStarListStarDict():
         )
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
+    body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -2053,29 +1886,9 @@ def getFunctionCallHelperKeywordsStarListStarDict():
         )
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
+    body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -2151,32 +1964,9 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
         )
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=called_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(variable=kw_variable, source_ref=internal_source_ref),
-        StatementReleaseVariable(
-            variable=star_arg_list_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=star_arg_dict_variable, source_ref=internal_source_ref
-        ),
-    )
+    body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=statements,
-                final=final,
-                source_ref=internal_source_ref,
-            )
-        )
-    )
+    result.setBody(body)
 
     return result
 
@@ -2218,7 +2008,7 @@ def getFunctionCallHelperDictionaryUnpacking():
                     ),
                     source_ref=internal_source_ref,
                 ),
-                right=makeExpressionBuiltinRef(
+                right=makeExpressionBuiltinTypeRef(
                     builtin_name="str", source_ref=internal_source_ref
                 ),
                 source_ref=internal_source_ref,
@@ -2251,7 +2041,7 @@ def getFunctionCallHelperDictionaryUnpacking():
                 variable=tmp_key_variable, source_ref=internal_source_ref
             ),
             value=ExpressionSubscriptLookup(
-                subscribed=ExpressionTempVariableRef(
+                expression=ExpressionTempVariableRef(
                     variable=tmp_item_variable, source_ref=internal_source_ref
                 ),
                 subscript=ExpressionTempVariableRef(
