@@ -19,6 +19,8 @@
 
 """
 
+import __future__
+
 import ast
 from logging import warning
 
@@ -58,7 +60,7 @@ def dump(node):
 
 
 def getKind(node):
-    return node.__class__.__name__.split(".")[-1]
+    return node.__class__.__name__.rsplit(".", 1)[-1]
 
 
 def extractDocFromBody(node):
@@ -71,7 +73,10 @@ def extractDocFromBody(node):
             doc = body[0].value.s
             body = body[1:]
         elif getKind(body[0].value) == "Constant":  # python3.8
-            doc = body[0].value.value
+            # Only strings should be used, but all other constants can immediately be ignored,
+            # it seems that e.g. Ellipsis is common.
+            if type(body[0].value.value) is str:
+                doc = body[0].value.value
             body = body[1:]
 
         if "no_docstrings" in Options.getPythonFlags():
@@ -106,9 +111,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
     flags = set()
 
     def _checkCoroutine(field):
-        """ Check only for co-routine nature of the field and only update that.
-
-        """
+        """Check only for co-routine nature of the field and only update that."""
         # TODO: This is clumsy code, trying to achieve what non-local does for
         # Python2 as well.
 
@@ -375,7 +378,13 @@ def buildAnnotationNode(provider, node, source_ref):
 
         _host_node.body[0].annotation = node
 
-        r = compile(_host_node, "<annotations>", "exec", 1048576, dont_inherit=True)
+        r = compile(
+            _host_node,
+            "<annotations>",
+            "exec",
+            __future__.CO_FUTURE_ANNOTATIONS,
+            dont_inherit=True,
+        )
 
         # Using exec here, to compile the ast node tree back to string,
         # there is no accessible "ast.unparse", and this works as a hack
@@ -383,6 +392,11 @@ def buildAnnotationNode(provider, node, source_ref):
         m = {}
         exec(r, m)
 
+        # TODO: In Python3.9, we should get https://bugs.python.org/issue35143
+        # to be used.
+        if Options.is_debug:
+            assert python_version <= 390
+
         return makeConstantRefNode(
             constant=m["__annotations__"]["x"], source_ref=source_ref
         )
@@ -479,7 +493,7 @@ def buildFrameNode(provider, nodes, code_object, source_ref):
 
 
 def makeStatementsSequenceOrStatement(statements, source_ref):
-    """ Make a statement sequence, but only if more than one statement
+    """Make a statement sequence, but only if more than one statement
 
     Useful for when we can unroll constructs already here, but are not sure if
     we actually did that. This avoids the branch or the pollution of doing it
@@ -632,7 +646,7 @@ def makeDictCreationOrConstant(keys, values, source_ref):
 
 def makeDictCreationOrConstant2(keys, values, source_ref):
     # Create dictionary node. Tries to avoid it for constant values that are not
-    # mutable. Keys are strings.
+    # mutable. Keys are Python strings here.
 
     assert len(keys) == len(values)
     for value in values:
